"""
    dyxi

    extract direct download links dynamically.

    :copyright: (c) 2021-2022 by Mahmoud Elshahat.
    :license: GNU LGPLv3, see LICENSE for more details.

    Module description:
        use webbrowser to get info via user interaction, e.g. extract download links which dynamically loaded by
        javascript.

"""

from threading import Thread
import time
from threading import Thread
from seleniumwire import webdriver
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities


def threaded(func):
    """a decorator to run any function / method in a thread
    you can pass threaded=False option when calling the decorated function if you need to disable threadding"""

    def wraper(*args, **kwargs):
        if kwargs.get('threaded', True):
            Thread(target=func, args=args, kwargs=kwargs, daemon=True).start()
        else:
            func(*args, **kwargs)

    return wraper


def extract_direct_link(url, q, pageLoadStrategy='none', headless=False, drivertype='chrome', webdriverfp=None,
                        browserbinaryfp=None):
    """start a controlled web-browser window, and monitor network trafficto catch urls

    Args:
        url(str): page url/link to request
        q(Queue): a queue to act as a container for caught urls, required for thread safety
        pageLoadStrategy(str): none normal, or eager,
                               see https://www.w3.org/TR/webdriver/#dfn-table-of-page-load-strategies
        headless(bool): if true it will not show web-browser window
        drivertype(str): either chrome or firefox
        webdriverfp(str): webdriver executable file path
        browserbinaryfp(str): web-browser executable file path

    """

    print(f'start webdriver, pageLoadStrategy: {pageLoadStrategy}, headless: {headless}')

    # setup
    if drivertype == 'chrome':
        from selenium.webdriver.chrome.service import Service
        options = webdriver.ChromeOptions()
        caps = DesiredCapabilities().CHROME
        driverfactory = webdriver.Chrome

    else:
        from selenium.webdriver.firefox.service import Service
        options = webdriver.FirefoxOptions()
        caps = DesiredCapabilities().FIREFOX
        driverfactory = webdriver.Firefox

    if browserbinaryfp:
        options.binary_location = browserbinaryfp

    options.headless = headless
    caps["pageLoadStrategy"] = pageLoadStrategy  # none normal, eager
    srv = Service(webdriverfp) if webdriverfp else Service()

    # initialize
    driver = driverfactory(desired_capabilities=caps, options=options, service=srv)

    Thread(target=driver.get, args=[url], daemon=False).start()

    urls = set()

    # extract link
    while True:
        for request in driver.requests:
            if request.url not in urls:
                print('url: ' + request.url)
                urls.add(request.url)
                q.put(request.url)

        try:
            # quit if user closed web-browser window
            driver.title
        except:
            break

        time.sleep(1)

    q.put(None)
    driver.quit()
