"""
    dyxi

    extract direct download links dynamically.

    :copyright: (c) 2021-2022 by Mahmoud Elshahat.
    :license: GNU LGPLv3, see LICENSE for more details.

    Module description:
        GUI, Graphical user interface by tkinter.

"""

import os
import platform
import json
import tkinter as tk
from queue import Queue
from threading import Thread
from tkinter import ttk
import awesometkinter as atk

from dyxi.extractor import threaded, extract_direct_link
from dyxi.version import __version__

APP_NAME = 'dyxi'
operating_system = platform.system()  # current operating system  ('Windows', 'Linux', 'Darwin')

fp = os.path.realpath(__file__)
cwd = os.path.dirname(fp)


# colors
# main colors
MAIN_BG = "#1c1c21"
MAIN_FG = "white"

BTN_BG = "#d9dc4b"
BTN_FG = "black"
BTN_HBG = "#000300"
BTN_ABG = "#000300"
BTN_AFG = "white"

# scrollbar
SBAR_BG = "#1c1c21"
SBAR_FG = "white"

# right click menu
RCM_BG = "#1c1c21"
RCM_FG = "white"
RCM_ABG = "#d9dc4b"
RCM_AFG = "black"


def get_global_sett_folder():
    """return a proper global setting folder"""
    home_folder = os.path.expanduser('~')

    if operating_system == 'Windows':
        roaming = os.getenv('APPDATA')  # return APPDATA\Roaming\ under windows
        _sett_folder = os.path.join(roaming, f'.{APP_NAME}')

    elif operating_system == 'Linux':
        _sett_folder = f'{home_folder}/.config/{APP_NAME}/'

    elif operating_system == 'Darwin':
        _sett_folder = f'{home_folder}/Library/Application Support/{APP_NAME}/'

    else:
        _sett_folder = cwd

    return _sett_folder


sett_folder = get_global_sett_folder()
configfp = os.path.join(sett_folder, 'config.json')
opt = {}

# load config
try:
    with open(configfp, 'r') as f:
        opt = json.load(f)
except:
    pass


def button(*args, **kwargs):
    kwargs.update(bg=BTN_BG, fg=BTN_FG, highlightbackground=BTN_HBG, activebackground=BTN_ABG, activeforeground=BTN_AFG)
    return tk.Button(*args, **kwargs)


def start_gui():
    window = tk.Tk()
    window.config(bg=MAIN_BG)

    global opt
    browserbinaryfpvar = tk.StringVar(value=opt.get('browserbinaryfp', ''))
    webdriverfpvar = tk.StringVar(value=opt.get('webdriverfp', ''))
    filter_var = tk.StringVar(value=opt.get('filter', 'mp4 m3u8'))

    def save_config(*args):
        opt['browserbinaryfp'] = browserbinaryfpvar.get()
        opt['webdriverfp'] = webdriverfpvar.get()
        opt['filter'] = filter_var.get()

    for var in (browserbinaryfpvar, webdriverfpvar, filter_var):
        var.trace_add('write', save_config)

    # window = tk.Toplevel(self.root, bg=MAIN_BG)

    window.title(f'dyxi - dynamic link extractor - {__version__}')
    help_msg = 'Press "Go" button to open controlled browser window \n' \
               'when you play a video, the direct download link will be displayed below \n' \
               'use filter to tweak displayed urls'

    fr = tk.Frame(window, bg=MAIN_BG)
    fr.pack(padx=5, pady=5, anchor='w')
    tk.Label(fr, text='Extract links dynamically:', justify='left', bg=MAIN_BG, fg=MAIN_FG).pack(side='left')
    # button(fr, text='hint!', command=lambda: self.popup(help_msg)).pack(side='left', padx=5)

    for txt, var in {'Web-browser executable: ': browserbinaryfpvar, 'web-driver executable: ': webdriverfpvar}.items():
        fr = tk.Frame(window, bg=MAIN_BG)
        fr.pack(fill='x')
        tk.Label(fr, text=txt, justify='left', bg=MAIN_BG, fg=MAIN_FG).pack(side='left', pady=5, padx=5)
        tk.Entry(fr, textvariable=var, bg=MAIN_BG, fg=MAIN_FG, insertbackground=MAIN_FG, bd=0
                 ).pack(side='left', fill='x', expand=True, padx=5)

    urlvar = tk.StringVar()

    @threaded
    def process():
        url = urlvar.get()
        browserfp = browserbinaryfpvar.get()
        driverfp = webdriverfpvar.get()
        kwargs = dict(webdriverfp=driverfp, browserbinaryfp=browserfp)
        q = Queue()
        Thread(target=extract_direct_link, args=(url, q), kwargs=kwargs).start()

        while True:
            d_url = q.get()
            if d_url is None:
                break
            else:
                update(d_url=d_url)

    urlfr = tk.Frame(window, bg=MAIN_BG)
    urlfr.pack(fill='x', pady=10)
    button(urlfr, text='Open web-browser', command=process).pack(side='left', padx=5)
    ttk.Separator(window, orient=tk.HORIZONTAL).pack(fill='x', pady=(0, 10))

    # filter
    fr2 = tk.Frame(window, bg=MAIN_BG)
    fr2.pack(fill='x')

    tk.Label(fr2, text='Filter urls with keywords: ', bg=MAIN_BG, fg=MAIN_FG).pack(side='left', padx=5)
    tk.Entry(fr2, textvariable=filter_var, bg=MAIN_BG, fg=MAIN_FG, justify='center',
             insertbackground=MAIN_FG).pack(side='left', fill='both')
    urls = []
    filter_kw = [x.strip() for x in filter_var.get().split()]

    def applyfilter():
        nonlocal filter_kw
        filter_kw = [x.strip() for x in filter_var.get().split()]
        filtered_urls = [url for url in urls if [z for z in filter_kw if z in url]] if filter_kw else urls
        txt.clear()
        txt.set('\n'.join(filtered_urls))

    def clearfilter():
        filter_var.set('')
        filter_kw.clear()
        txt.clear()
        txt.set('\n'.join(urls))

    def clear_urls():
        urls.clear()
        txt.clear()

    button(fr2, text='Apply filter', command=applyfilter).pack(side='left', padx=5)
    button(fr2, text='Clear filter', command=clearfilter).pack(side='left', padx=5)
    button(fr2, text='Clear urls', command=clear_urls).pack(side='right', padx=5)

    txt = atk.ScrolledText(window, height=4, bg=MAIN_BG, fg=MAIN_FG, sbar_bg=SBAR_BG, sbar_fg=SBAR_FG, padx=5, bd=2)
    txt.pack(expand=True, fill='both', pady=5)
    txt.set('\n'.join(urls))

    atk.RightClickMenu(txt, ['Cut', 'Copy', 'Paste'],
                       callback=lambda selected, w=txt: w.event_generate(f'<<{selected}>>'),
                       bg=RCM_BG, fg=RCM_FG, afg=RCM_AFG, abg=RCM_ABG)

    def update(d_url=None):
        if d_url and d_url not in urls:
            urls.append(d_url)
            if [z for z in filter_kw if z in d_url] or not filter_kw:
                txt.append(d_url + '\n')

    window.update = update

    btnfr = tk.Frame(window, bg=MAIN_BG)
    btnfr.pack(fill='x', pady=10)

    button(btnfr, text='Close', command=lambda: window.destroy()).pack(side='right', padx=5)

    window.minsize(50, 50)

    refx = window.winfo_screenwidth() // 2
    refy = window.winfo_screenheight() // 2

    width, height = refx, refy

    x = refx - width // 2
    y = refy - height // 2

    window.geometry(f'{width}x{height}+{x}+{y}')

    window.mainloop()


def main():
    start_gui()

    # save config
    try:
        os.makedirs(sett_folder, exist_ok=True)
        with open(configfp, 'w') as f:
            json.dump(opt, f)
    except:
        raise


if __name__ == '__main__':
    main()
